@@ -1,3 +1,4 @@
+import logging
 import math
 import os
 import pathlib
@@ -7,6 +8,7 @@ from hashlib import sha256
 from urllib.parse import urlsplit
 
 DEFAULT_BLOCK_SIZE = 5 * 2 ** 20
+PY36 = sys.version_info < (3, 7)
 
 
 def infer_storage_options(urlpath, inherit_storage_options=None):
@@ -69,7 +71,7 @@ def infer_storage_options(urlpath, inherit_storage_options=None):
         # https://github.com/dask/dask/issues/1417
         options["host"] = parsed_path.netloc.rsplit("@", 1)[-1].rsplit(":", 1)[0]
 
-        if protocol in ("s3", "gcs", "gs"):
+        if protocol in ("s3", "s3a", "gcs", "gs"):
             options["path"] = options["host"] + options["path"]
         else:
             options["host"] = options["host"]
@@ -115,7 +117,7 @@ def infer_compression(filename):
     extension. This includes builtin (gz, bz2, zip) compressions, as well as
     optional compressions. See fsspec.compression.register_compression.
     """
-    extension = os.path.splitext(filename)[-1].strip(".")
+    extension = os.path.splitext(filename)[-1].strip(".").lower()
     if extension in compressions:
         return compressions[extension]
 
@@ -305,7 +307,10 @@ def stringify_path(filepath):
         return filepath.__fspath__()
     elif isinstance(filepath, pathlib.Path):
         return str(filepath)
-    return filepath
+    elif hasattr(filepath, "path"):
+        return filepath.path
+    else:
+        return filepath
 
 
 def make_instance(cls, args, kwargs):
@@ -384,23 +389,6 @@ def can_be_local(path):
         return False
 
 
-def setup_logger(logname, level="DEBUG", clear=True):
-    """Add standard logging handler to logger of given name"""
-    import logging
-
-    logger = logging.getLogger(logname)
-    if clear:
-        logger.handlers.clear()
-    handle = logging.StreamHandler()
-    formatter = logging.Formatter(
-        "%(asctime)s - %(name)s - %(levelname)s " "- %(message)s"
-    )
-    handle.setFormatter(formatter)
-    logger.addHandler(handle)
-    logger.setLevel(level)
-    return logger
-
-
 def get_package_version_without_import(name):
     """For given package name, try to find the version without importing it
 
@@ -419,14 +407,14 @@ def get_package_version_without_import(name):
             import importlib.metadata
 
             return importlib.metadata.distribution(name).version
-        except ImportError:
+        except:  # noqa: E722
             pass
     else:
         try:
             import importlib_metadata
 
             return importlib_metadata.distribution(name).version
-        except ImportError:
+        except:  # noqa: E722
             pass
     try:
         import importlib
@@ -435,3 +423,19 @@ def get_package_version_without_import(name):
         return mod.__version__
     except (ImportError, AttributeError):
         return None
+
+
+def setup_logging(logger=None, logger_name=None, level="DEBUG", clear=True):
+    if logger is None and logger_name is None:
+        raise ValueError("Provide either logger object or logger name")
+    logger = logger or logging.getLogger(logger_name)
+    handle = logging.StreamHandler()
+    formatter = logging.Formatter(
+        "%(asctime)s - %(name)s - %(levelname)s - %(funcName)s -- %(message)s"
+    )
+    handle.setFormatter(formatter)
+    if clear:
+        logger.handlers.clear()
+    logger.addHandler(handle)
+    logger.setLevel(level)
+    return logger
